package com.lanren.huhu.partner.manager;

import com.lanren.huhu.partner.domain.UserAgent;
import com.lanren.huhu.partner.model.AgentResponse;
import com.lanren.huhu.partner.model.ParentAgent;
import com.lanren.huhu.partner.result.Result;
import com.lanren.huhu.partner.schedule.AgentSettleTask;
import com.lanren.huhu.partner.service.UserAgentService;
import com.lanren.huhu.partner.service.UserService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.ArrayList;

import static com.lanren.huhu.partner.constants.Constants.PARENT_COLUMN_NAME_CENGJI;
import static com.lanren.huhu.partner.constants.Constants.PARENT_COLUMN_NAME_PINGTUI;

/**
 * @author chen
 */
@Component
public class AgentManager {
    private static Logger logger = LoggerFactory.getLogger(AgentManager.class);

    @Autowired
    UserAgentService userAgentService;
    @Autowired
    AgentSettleTask agentSettleTask;
    @Autowired
    UserService userService;

    /***
     * * 获取代理商等级 1 总代, 2 大区, 3 省, 4 市
     * @param agentId
     * @return
     */
    public Result<AgentResponse> getAgentLevel(Integer agentId) {
        if (logger.isDebugEnabled()) {
            logger.info("UserAgentManager getLevel agentId:{},time:{}", agentId, LocalDateTime.now());
        }
        Result<AgentResponse> result = new Result<>();
        UserAgent userAgent = userAgentService.getOneByAgentId(agentId);
        if (null != userAgent) {
            result.setData(new AgentResponse(userAgent.getUserId(), userAgent.getAgentLevel()));
        } else {
            result.setCode(601);
        }
        return result;
    }

    /**
     * @description: 获取一个代理商的层级上级
     * @author chen
     * @date 2019-07-04 14:48
     */
    public Result<ArrayList<ParentAgent>> getHierarchyParentList(Integer agentId) {
        UserAgent userAgent = userAgentService.getOneByAgentId(agentId);
        ArrayList<ParentAgent> parentAgents = (ArrayList<ParentAgent>) userService.getAgentListByUserId(userAgent.getUserId(), PARENT_COLUMN_NAME_CENGJI);
        Result<ArrayList<ParentAgent>> result = new Result<ArrayList<ParentAgent>>();
        result.setData(parentAgents);
        return result;
    }
    /**
     * @description: 获取一个代理商的平推上级
     * @author chen
     * @date 2019-07-04 14:48
     */
    public Result<ArrayList<ParentAgent>> getPingtuiParentList(Integer agentId) {
        UserAgent userAgent = userAgentService.getOneByAgentId(agentId);
        ArrayList<ParentAgent> parentAgents = (ArrayList<ParentAgent>) userService.getAgentListByUserId(userAgent.getUserId(), PARENT_COLUMN_NAME_PINGTUI);
        Result<ArrayList<ParentAgent>> result = new Result<ArrayList<ParentAgent>>();
        result.setData(parentAgents);
        return result;
    }

    public Result<String> doFinanceSettle(int agentId) {
        Result<String> result = new Result<String>();
        if (agentSettleTask.doOneAgentSettle(agentId)) {
            result.setCode(0);
        } else {
            result.setCode(500);
        }
        return result;
    }
}
